;***************************************************************************
;
; File Name		:'Boot_XModem.asm"
; Title			:XMODEM receive only for BOOTUpgrade
; Date			:2002.07.03.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034, Old: +36-30-9541-658
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:AVR
;
;***************************************************************************
;	D E S C R I P T I O N
;
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  why
;	----	----------  ---	--------------------------------------------
;	0.01	2002.07.03  VFX	Creation
;	0.02	2006.04.28  VFX	Redesign for M88 BootUpgr.
;
;***************************************************************************
;* Const Def

.EQU	Xmodem_Sync	=	0	;szinkronizalasi fazis
.EQU	Xmodem_SOH	=	1	;SOH byte-ra var
.EQU	Xmodem_Fill	=	2	;SOH byte megjott, block init megvolt
.EQU	Xmodem_Err	=	3	;megszakadt az atvitel, NAK kuldese 3 masodpercenkent


.EQU	SyncTiming	=	25	;ket karakter elkuldese kozott defaullt =300
					;eltelt ido szinc. uzemmodban 3sec @100Hz

.equ	MaxError	=	20

;**************************************************************************
;* Hardware Def.
;


;***************************************************************************
;**** VARIABLES
.DSEG


CRC:		.BYTE	2		;CRC for XMODEM
XmodemFlag:	.BYTE	1		;Xmodem Flag
					;[2:0] = 000 sending 'c' Phase 1: syncronize

Xm_BlkCounter:	.BYTE	1		;Xmodem Block Counter [00-FF]
XmodemPTR:	.BYTE	1		;Xmodem Buffer pointer
XmodemBuf:	.BYTE	133		;XMODEM RxBuffer
LastPocketSize:	.byte	1		;utolso pocket merete byte-ban [0...128]
XmodemMode:	.byte	1		;Mode0 - Nul modem letolti de nem csinal semmit
					;Mode1 - DSP Boot
Xm_ErrorCNT:	.byte	1		;Xmodem Time to Abort counter

;***************************************************************************
.ESEG


;***************************************************************************
.CSEG


;***************************************************************************
;** Init Xmodem File Receive
;**

Xmodem_Init:
		ldi	R16,128
		sts	LastPocketSize,R16
		clr	R0
		sts	Xm_ErrorCNT,R0
		sts	XmodemFlag,R0
		sts	Xm_BlkCounter,R0
		inc	R0
		sts	XmodemMode,R0
Init_XmNextBl:	clr	R0
		sts	CRC+0,R0
		sts	CRC+1,R0
		sts	XmodemPTR,R0
		sts	SCNT2+0,R0
		sts	SCNT2+1,R0
		ret


;***************************************************************************
;** Calc & Update XMODEM CRC
;*
;* In: R0 - data byte

UpdateCRC:      lds     R17,CRC+1
		mov     R15,R17
		swap    R17
		mov     R16,R17

		eor     R17,R15
		andi    R17,0xF0
		andi    R16,0x0F
		eor     R15,R16

		mov     R14,R17
		lsl     R14
		rol     R16

		eor     R15, R14
		eor     R17,R16

		lds     R14,CRC+0
		eor     R15,R0
		eor     R17,R14

		sts     CRC+0,R15
		sts     CRC+1,R17
		ret


;***************************************************************************
;** XMODEM Receive File & Block
;*
;*
Xmodem_Receive:

Xmodem_Rec:
	    	lds	R16,XmodemFlag
		cpi	R16,Xmodem_Sync
		breq	Xm_Sync

		cpi	R16,Xmodem_Fill
		brne	Xmodem_Rec1
		rjmp	Xm_Fill

Xmodem_Rec1:	cpi	R16,Xmodem_SOH
		breq	Xm_SOH

		cpi	R16,Xmodem_Err
		breq	Xm_Sync

		rjmp	Xmodem_Cancel


Xm_Sync:	rcall	Rs232GetByte
		brcs	Xm_valamijott
						;itt meg mindig nem jott semmi
		lds	R16,SCNT2+0
		lds	R17,SCNT2+1
		or	R16,R17			;Timer expired?
		breq	Xm_Sync1
		ret

Xm_Sync1:	ldi	R16,NAK			;akkor, ha hiba van  NAK megy 3 masodpercenkent
		lds	R17,XmodemFlag
		cpi	R17,Xmodem_Err
		breq	Xm_Sync2
						;mivel meg mindig nem jott semmi kuldunk egy
						;CRC - karaktert 3 masodpercenkent

		lds	R16,Xm_ErrorCNT		;ha MaxError 'C' ideig nem jon semmi akkor abort!!!
		inc	R16
		sts	Xm_ErrorCNT,R16
		cpi	R16,MaxError
		brne	Xm_Sync25
		rjmp	Xm_Abort

Xm_Sync25:	ldi	R16,'C'


Xm_Sync2:	rcall	SendChrW


;		rcall	TogleChgLED


		ldi	R16,Low(SyncTiming)	;Timer=3sec @100Hz
		sts	SCNT2+0,R16
		ldi	R17,High(SyncTiming)
		sts	SCNT2+1,R17
		ret

Xm_SOH:		rcall	Rs232GetByte
		brcs	Xm_valamijott
		ret

Xm_valamijott:

;		sbi	LED_CHG_PORT,LED_CHG

		mov	R16,R0
		cpi	R16,CAN
		breq	Xmodem_Cancel
		cpi	R16,SOH
		breq	Xmodem_StartOfH
		cpi	R16,EOT
		breq	Xmodem_EndOfT
                rjmp	Xmodem_Rec
						;End of Trans == Canacel
Xmodem_EndOfT:
		ldi	R16,ACK
		rcall	SendChrW		;Pozitiv nyugta
Xm_Abort:
		ldi	R16,CR
		rcall	SendChrW
Xmodem_Cancel:
Xm_CancelEnd:
		clr	R17
		sts	Updating,R17		;Ezel jelzzuk keszen vagyunk
		sts	XmodemMode,R17		;barmi volt is, most null modem
		ret				;rjmp	CMDOk


Xmodem_StartOfH:
		ldi	R16,Xmodem_Fill
		sts	XmodemFlag,R16
		rcall	Init_XmNextBl
		ldi	ZL,Low(SyncTiming)	;Timer=3sec @100Hz
		sts	SCNT2+0,ZL		;he nem jon ujjab byte, akkor baj van!!
		ldi	ZL,High(SyncTiming)
		sts	SCNT2+1,ZL
		rjmp	Xmodem_Rec


Xm_Fill:	rcall	Rs232GetByte		;itt beolvasunk 132 byte-ot
		brcs	Xm_data			; blk counter,255-blk counter [2 byte]
						; adatblok [128 byte]
						; CRC [2 byte]
						;total 132 byte

		lds	R16,SCNT2+0		;ha nem jon ujjab byte, akkor baj van!!
		lds	R17,SCNT2+1
		or	R16,R17
		breq	Xm_TimeOut
		ret				;Itt meg van ido, hogy jojjon valami
Xm_TimeOut:
		ldi	R16,Xmodem_Err
		sts	XmodemFlag,R16		;Time Out, Ismetles kell
		rjmp	Xmodem_Rec

Xm_data:
		ldi	ZL,Low(SyncTiming)	;Timer=3sec @100Hz
		sts	SCNT2+0,ZL		;ha nem jon ujjab byte, akkor baj van!!
		ldi	ZL,High(SyncTiming)
		sts	SCNT2+1,ZL

		ldi	ZL,Low(XmodemBuf)
		ldi	ZH,High(XmodemBuf)
		clr	R1
		lds	R16,XmodemPTR
		add	ZL,R16
		adc	ZH,R1			;ZL:ZH -> Xmodem Actualis Pos.
		st	Z,R0			;Char eltarolasa
		inc	R16
		sts	XmodemPTR,R16
         	cpi	R16,3			;3<Z<130 akkor kell CRC
		brcs	XmNoCRCUpdate
		push	R16
		rcall	UpdateCRC
		pop	R16
XmNoCRCUpdate:	cpi	R16,132
		brcs	Xm_Fill

                              			;itt jon a block ellenorzese
Xm_ProcessBlock:
             	lds	R16,XmodemBuf+0
		lds	R17,XmodemBuf+1
		add	R17,R16
		inc	R17
		breq	Xm_Proc1

Xm_Proc0:	ldi	R16,NAK
		rcall	SendChrW		;negativ nyugta, block count nem jo
		ldi	R16,Xmodem_SOH
		sts	XmodemFlag,R16
                rjmp	Xmodem_Rec

Xm_Proc1:
		lds	R0,Xm_BlkCounter
		inc	R0
		cp	R0,R16
		brne	Xm_Proc0		;block count nem az jott amit vartunk
		sts	Xm_BlkCounter,R0

		lds	R16,CRC+0
		lds	R17,CRC+1
		or	R16,R17
		brne	Xm_Proc0		;CRC hiba


		lds	R17,XmodemMode
		cpi	R17,0
		breq	Xm_NullModem
		cpi	R17,1
		brne	Xm_NullModem


		ldi	XL,Low(XmodemBuf+2)	;fejlec kimarad
		ldi	XH,High(XmodemBuf+2)
		ldi	R16,128			;max block hossz=128
		rcall	CopyAppData		;Copy to FLASH

Xm_NullModem:	ldi	R16,ACK
		rcall	SendChrW		;Pozitiv nyugta
		ldi	R16,Xmodem_SOH
		sts	XmodemFlag,R16
                rjmp	Xmodem_Rec
